#!/bin/bash
updateDirectory=$(cd "$(dirname "${BASH_SOURCE[0]}")" && pwd)
. $updateDirectory/scripts/common.sh

if [[ $EUID -ne 0 ]]; then
	print_error "This script must be run as root" 1>&2
	exit 1
fi

# Install prerequisite packages and software
ID=linux
VERSION_ID=1
if [ -f /etc/os-release ]; then
	. /etc/os-release
else
	print_error "Unable to detect file '/etc/os-release', assuming generic installation."
fi
if [ "$ID" == "ubuntu" ]; then
	if [ "$VERSION_ID" == "12.10" ]; then
		# 12.10 ubuntu images require their package sources to be fixed
		sed -i -e 's/us.archive/old-releases/g' /etc/apt/sources.list
		# Delete the security repo if it exists and then add it back (it isn't there by default, but if someone re-runs this installer it might be, don't want duplicates.)
		sed -i '\|deb http://security.ubuntu.com/ubuntu precise-security main|d' /etc/apt/sources.list
		echo "deb http://security.ubuntu.com/ubuntu precise-security main" >> /etc/apt/sources.list
	fi
	if [ "$VERSION_ID" == "13.04" ]; then
		# 13.04 ubuntu images require their package sources to be fixed
		sed -i -e 's/us.archive/old-releases/g' /etc/apt/sources.list
		sed -i -e 's/archive/old-releases/g' /etc/apt/sources.list
		# Install version-specific dependencies
		apt-get update
		apt-get install -f -y
		apt-get install -y --force-yes libimlib2-dev libtiff5-dev libx11-dev libxext-dev libtiff5=4.0.2-4ubuntu2 libx11-6=2:1.5.0-1ubuntu1 libxcb1-dev libxcb1=1.8.1-2ubuntu2 libxext6=2:1.3.1-2 libxfixes-dev libxfixes3=1:5.0-4ubuntu5 libxrandr-dev libxrandr2=2:1.4.0-1ubuntu1 libxrender-dev libxrender1=1:0.9.7-1
	fi
	if [ "$VERSION_ID" == "13.10" ]; then
		# 13.10 ubuntu images require their package sources to be fixed
		sed -i -e 's/us.archive/old-releases/g' /etc/apt/sources.list
		sed -i -e 's/archive/old-releases/g' /etc/apt/sources.list
		# Install version-specific dependencies
		apt-get update
		apt-get install -f -y
		apt-get install -y --force-yes libjpeg-turbo8=1.3.0-0ubuntu1
		apt-get install -y --force-yes libtiff5=4.0.2-4ubuntu3
		apt-get install -y --force-yes libjbig0=2.0-2ubuntu1
		apt-get install -y --force-yes libfreetype6=2.4.12-0ubuntu1
	fi
	if [ "$VERSION_ID" == "12.04" ]; then
		# Install version-specific dependencies
		apt-get update
		apt-get install -f -y
		apt-get install -y --force-yes libjpeg-turbo8=1.1.90+svn733-0ubuntu4
		apt-get install -y --force-yes libtiff4=3.9.5-2ubuntu1
		apt-get install -y --force-yes libx11-6=2:1.4.99.1-0ubuntu2
		apt-get install -y --force-yes libxcb1=1.8.1-1
		apt-get install -y --force-yes libxext6=2:1.3.0-3build1
		apt-get install -y --force-yes libfreetype6=2.4.8-1ubuntu2
		apt-get install -y --force-yes libxrandr2=2:1.3.2-2
		apt-get install -y --force-yes libxrender1=1:0.9.6-2build1
		apt-get install -y --force-yes libxfixes3=1:5.0-4ubuntu4
	fi
fi
#if [ `uname -m` == "armv7l" ]; then
	# There are no prebuilt binaries for ARM, so we must manually build them.
	# https://github.com/LeonardLaszlo/nw.js-armv7-binaries/releases/download/nwjs-sdk-v0.17.5-linux-ARMv7/nwjs-sdk-v0.17.5-linux-arm.tar.gz
#fi

# Install required packages
print_info "Attempting to install required packages..."
apt-get update
apt-get install -f -y
apt-get install -y libnss3 libxss1 git cmake libimlib2-dev libxrandr-dev libxfixes-dev unzip xdotool g++

# Build and install "maim" (screenshot utility)
if ! [ -x "$(command -v maim)" ]; then
	(
	cd /tmp
	print_info "Downloading maim source code..."
	wget https://internal.keywesttechnology.com/breeze2/mirror/maim.tar.gz
	tar xf ./maim.tar.gz
	# git clone https://github.com/naelstrof/maim.git
	cd maim
	# git checkout 4526cddb9912229a5e6ec97c497d3288cab027c7
	cmake ./
	make && make install
	print_info "Cleaning up temporary maim files..."
	rm -rf /tmp/maim
	rm -f /tmp/maim.tar.gz
	)
fi

# Create required directories
rm -rf $rootPath
mkdir -p $rootPath

CopyFiles
#InstallNode
InstallNwjs
CreateLogs
CreateWatchdog
CreateShortcuts

# Set permissions
print_info "Setting permissions..."
chown -R $user:$user $rootPath
chmod -R 777 $rootPath

sudoersPath=/etc/sudoers.d/breeze2
cat > $sudoersPath << EOL
$user ALL = NOPASSWD: ALL
EOL
chmod 0440 $sudoersPath

print_info "Finished, please check the above output for errors."
