function FormatPath ([String]$path) {
	# https://stackoverflow.com/questions/3038337/powershell-resolve-path-that-might-not-exist/12605755
	# Old code was this, but I'm not sure it's the most valid way
	# [IO.Path]::GetFullPath($path)
	return $ExecutionContext.SessionState.Path.GetUnresolvedProviderPathFromPSPath($path)
}

function SaveFile([String]$path, [String]$text) {
	$text = $text -replace "`n","`r`n"
	Out-File -Filepath $path -InputObject $text -Encoding ASCII
}

function DownloadRemoteFile ([String]$fetchUrl, [String]$destinationFilepath) {
	Write-Output "Forcing SecurityProtocol to Tls12.."
	[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::Tls12

	$startTime = Get-Date

	$maxAttempts = 5
	$retryIntervalSeconds = 5 * 60
	$attempt = 1

	while ($attempt -le $maxAttempts) {
		try {
			Write-Output "Starting download from '$fetchUrl', saving to '$destinationFilepath' attempt $attempt / $maxAttempts..."
			Write-Output "This may take a few minutes... do not close!"

			$client = New-Object System.Net.WebClient
			$client.Headers.Add("user-agent", "Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/200.0.0.0 Safari/537.36")
			$client.Proxy.Credentials = [System.Net.CredentialCache]::DefaultNetworkCredentials
			$client.DownloadFile($fetchUrl, $destinationFilepath)
			break
		} catch {
			Write-Output "An error occurred: $_"
			Write-Output $_.ScriptStackTrace
			Write-Output $Error[0].Exception.ToString()
			if ($attempt -lt $maxAttempts) {
				Write-Output "Retrying in $retryIntervalSeconds seconds..."
				Start-Sleep -Seconds $retryIntervalSeconds
			} else {
				Write-Output "Maximum attempts reached. Exiting."
				throw "Download $fetchUrl failed $maxAttempts times. Critical error, usually networking related."
			}
		}
		$attempt++
	}
	$endTime = Get-Date
	$totalSeconds = $(New-Timespan $startTime $endTime).TotalSeconds
	Write-Output "Download finished in $totalSeconds seconds.."
	Write-Output "Resetting SecurityProtocol.."
	[System.Net.ServicePointManager]::SecurityProtocol = [System.Net.SecurityProtocolType]::SystemDefault
}

function ExtractZipFile ([String]$filePath, [String]$destinationPath) {
	# Convert the filepaths into "full" filepaths
	$file = [IO.Path]::GetFullPath($filePath)
	$dest = [IO.Path]::GetFullPath($destinationPath)
	Write-Output "Extracting Zip File $file to $dest"
	$Shell = New-Object -ComObject Shell.Application
	$fileHandle = $Shell.NameSpace($file)
	$destHandle = $Shell.NameSpace($dest)
	foreach ($item in $fileHandle.Items()) {
		$destHandle.CopyHere($item, 1044)
		$filename = $item.Name
		Write-Output "..\$filename moved to $dest\$filename.."
	}
}

function CopyFiles {
	# Make sure we're not in a directory or it gets locked on old Windows
	Set-Location -Path $rootPath
	
	try {
		# This lets us run these powershell scripts in a powershell window to debug
		Set-ExecutionPolicy Unrestricted -force -Scope CurrentUser
	} catch {
		Write-Output $_
	}
	
	# Note: Copy-Item has really weird behavior when copying to a directory that exists. Don't use it for folders
	# Note on Windows 10 1607 Remove-Item with -Recurse is broken, thus this weird script. This was fixed in like 2020.
	Write-Output "Copying Breeze container files..."
	Write-Output "Delete all the files except the config folder"

	# Try 3 times to delete the files. This code is not stable. Remove-Item is a mess in old windows.
	for ($i = 0; $i -le 3; $i++) {
		$filesWithoutConfig = Get-ChildItem $rootPath -Exclude config | Get-ChildItem -Recurse | Sort-Object -Property FullName -Descending
		if ($filesWithoutConfig -eq $null) {
			break
		}
		$filesWithoutConfig | Remove-Item -Recurse -Force
		Start-Sleep -Milliseconds 500
	}
	Get-ChildItem $rootPath -Exclude config | Remove-Item -Recurse -Force
	$updateDirectory = FormatPath "$PSScriptRoot\.."
	Write-Output "Copy the update files from $updateDirectory to $rootPath"
	cmd.exe /c "robocopy $updateDirectory $rootPath /E /NS /NC /NFL /NDL /NP /NJH /NJS"
	# Remove the version folder with the migration scripts. The -Recurse usage is because Include apparently is special
	Write-Output "Cleaning up migration version folder in $rootPath\0*"
	for ($i = 0; $i -le 3; $i++) {
		$oldVersionFolder = Get-ChildItem $rootPath -Filter 0* | Get-ChildItem -Recurse | Sort-Object -Property FullName -Descending
		if ($oldVersionFolder -eq $null) {
			break
		}
		$oldVersionFolder | Remove-Item -Recurse -Force
		Start-Sleep -Milliseconds 500
	}
	Get-ChildItem $rootPath -Filter 0* | Remove-Item -Recurse -Force
	# Remove any old update files
	Write-Output "Removing update files $configDirectory\updates\*"
	#try {
	#	Get-ChildItem $configDirectory\updates -Recurse | Sort-Object -Property FullName -Descending | Remove-Item -Force
	#} catch {
	#	Write-Output "Problem removing update files, usually caused by old windows"
	#}
	Write-Output "CopyFiles done"
}

function CreateShortcut ([String]$shortcutPath, [String]$targetPath, [String]$arguments, [String]$iconPath) {
	$WshShell = New-Object -ComObject WScript.Shell
	$desktopShortcut = $WshShell.CreateShortcut($shortcutPath)
	$desktopShortcut.TargetPath = $targetPath
	$desktopShortcut.Arguments = $arguments
	$desktopShortcut.IconLocation = $iconPath
	$desktopShortcut.Save()
}

function OptimizingPowershell () {
	Write-Output "Optimizing powershell"
	$env:PATH = [Runtime.InteropServices.RuntimeEnvironment]::GetRuntimeDirectory()
	[AppDomain]::CurrentDomain.GetAssemblies() | ForEach-Object {
		$path = $_.Location
		if ($path) {
			$name = Split-Path $path -Leaf
			Write-Host -ForegroundColor Yellow "`r`nRunning ngen.exe on '$name'"
			ngen.exe install $path /nologo
		}
	}
}

function WrapPowerShellInVbs([String]$ps1Path, [String]$vbsPath) {
	# This must use WShell. PowerShell is a console and there is no way to hide it without it flashing for a second. (This is a known issue and MS isn't going to fix it).
	SaveFile $vbsPath @"
Dim WShell
Set WShell = CreateObject("WScript.Shell")
WShell.Run "powershell.exe -nologo -command $ps1Path", 0
Set WShell = Nothing
"@
}

# Create a watchdog to kill explorer.exe if nw has been running 15 minutes else it starts it
function CreateExplorerWatchdog {
	SaveFile $explorerWatchdogPs1Path @"
for (`$i = 0; `$i -lt 6; `$i++) {
	`$nwProcess = Get-Process -Name "nw" -ErrorAction SilentlyContinue
	`$explorerProcess = Get-Process -Name "explorer" -ErrorAction SilentlyContinue

	if (`$nwProcess) {
		`$nwStartTime = `$nwProcess.StartTime

		if (`$nwStartTime -lt (Get-Date).AddMinutes(-15)) {
			if (`$explorerProcess) {
				taskkill /F /IM explorer.exe
			}
		}
	} else {
		if (-not `$explorerProcess) {
			Start-Process explorer
		}
	}
	Start-Sleep -Seconds 8
}
"@

	WrapPowerShellInVbs $explorerWatchdogPs1Path $explorerWatchdogVbsPath

	# Start the scheduled task
	$action = New-ScheduledTaskAction -Execute $explorerWatchdogVbsPath
	$principal = New-ScheduledTaskPrincipal -RunLevel Highest -UserId "$env:USERDOMAIN\$env:USERNAME"
	# The -1 year is because of an issue with players updating in the wrong TimeZone or with the wrong time
	$trigger = New-ScheduledTaskTrigger -Once -At (Get-Date).AddYears(-1) -RepetitionInterval (New-TimeSpan -Minutes 1)
	$task = New-ScheduledTask -Action $action -Trigger $trigger -Principal $principal
	Register-ScheduledTask -TaskName "$explorerWatchdogTaskName" -InputObject $task -Force
}

function CreateWatchdog {
	SaveFile $watchdogPs1Path @"
if (Test-Path $updateLockPath) {
	exit
}
# Very important. Ensure that Start-Process (and thus the nw process) is started in this directory
cd $rootPath
# If the nw.exe is not running or is using less than 15MB of memory kill it and restart
# Starting in 0.49.0 the nwjs has a bug that leaves orphaned nw.exe instances running so this 15MB check fixes that
`$nw = Get-Process nw -ErrorAction SilentlyContinue
if (`$nw -eq `$null -Or (Get-Process | Sort-Object -Descending WS | Where {`$_.Name -eq "nw"} | Select -First 1 -ExpandProperty WS) -lt 15000000) {
	Stop-Process -Name "nw" -ErrorAction SilentlyContinue
	Start-Sleep -s 1
	("`$("[{0:MM/dd/yy} {0:HH:mm:ss}]" -f (Get-Date)) | Detected container was not running... launching container...") >> $runlogPath
	Start-Process -FilePath $rootPath/nw.exe -RedirectStandardError "$errorlogPath"
}
"@

	WrapPowerShellInVbs $watchdogPs1Path $watchdogVbsPath
	
	# Start the scheduled task
	$action = New-ScheduledTaskAction -Execute $watchdogVbsPath
	$principal = New-ScheduledTaskPrincipal -RunLevel Highest -UserId "$env:USERDOMAIN\$env:USERNAME"
	# The -1 year is because of an issue with players updating in the wrong TimeZone or with the wrong time
	$trigger = New-ScheduledTaskTrigger -Once -At (Get-Date).AddYears(-1) -RepetitionInterval (New-TimeSpan -Minutes 1)
	$task = New-ScheduledTask -Action $action -Trigger $trigger -Principal $principal
	Register-ScheduledTask -TaskName "$watchdogTaskName" -InputObject $task -Force

	SaveFile $stopMonitorPath @"
Stop-Process -Name "nw" -ErrorAction SilentlyContinue
Start-Sleep -s 1
Unregister-ScheduledTask -TaskName "$watchdogTaskName" -Confirm:`$false
"@

	SaveFile $stopMonitorBatPath @"
$env:SystemRoot\System32\WindowsPowerShell\v1.0\powershell.exe -NoProfile -ExecutionPolicy Bypass -File $stopMonitorPath
"@

	SaveFile $startMonitorPath @"
Stop-Process -Name "nw" -ErrorAction SilentlyContinue
Start-Sleep -s 1

# Schedule the watchdog to run every minute
`$action = New-ScheduledTaskAction -Execute "$watchdogVbsPath"
`$principal = New-ScheduledTaskPrincipal -RunLevel Highest -UserId "$env:USERDOMAIN\$env:USERNAME"
`$trigger = New-ScheduledTaskTrigger -Once -At (Get-Date).AddYears(-1) -RepetitionInterval (New-TimeSpan -Minutes 1)
`$task = New-ScheduledTask -Action `$action -Trigger `$trigger -Principal `$principal
Register-ScheduledTask -TaskName "$watchdogTaskName" -InputObject `$task -Force

# Run the watchdog once
Start-Process -WindowStyle Hidden PowerShell.exe -Args "-WindowStyle Hidden $watchdogPs1Path"
"@

	SaveFile $startMonitorBatPath @"
$env:SystemRoot\System32\WindowsPowerShell\v1.0\powershell.exe -NoProfile -ExecutionPolicy Bypass -File $startMonitorPath
"@

	# Create a task that starts the watchdog monitoring on reboot (in case someone stopped it and forgot about it.)
	$action = New-ScheduledTaskAction -Execute "PowerShell.exe" -Argument "-windowstyle hidden $startMonitorPath"
	$principal = New-ScheduledTaskPrincipal -RunLevel Highest -UserId "$env:USERDOMAIN\$env:USERNAME"
	$trigger = New-ScheduledTaskTrigger -AtLogon
	$task = New-ScheduledTask -Action $action -Trigger $trigger -Principal $principal
	Register-ScheduledTask -TaskName "$rebootTaskName" -InputObject $task -Force

	CreateExplorerWatchdog
}

function CreateShortcuts {
	# Double clicking the shortcut opens notepad by default. Switch Windows to execute them instead
	Set-Itemproperty -path "Registry::HKEY_CLASSES_ROOT\Microsoft.PowerShellScript.1\Shell" -Name "(Default)" -value "0"

	Remove-Item -Path $desktopShortcutPath -Force

	$stopBreezeIconPath = FormatPath "$rootPath\icons\breeze_monitor_stop_icon.ico"
	CreateShortcut $stopBreezeShortcutPath $stopMonitorBatPath "" $stopBreezeIconPath
	ForceRequireAdmin $stopBreezeShortcutPath

	$startBreezeIconPath = FormatPath "$rootPath\icons\breeze_monitor_start_icon.ico"
	CreateShortcut $startBreezeShortcutPath $startMonitorBatPath "" $startBreezeIconPath
	ForceRequireAdmin $startBreezeShortcutPath
}

# This won't work for Powershell scripts that we have above! Powershell scripts are not executable.
function ForceRequireAdmin ([String]$shortcutPath) {
	# Credit to the answer by Jan Chrbolka: http://stackoverflow.com/questions/28997799/how-to-create-a-run-as-administrator-shortcut-using-powershell
	$bytes = [System.IO.File]::ReadAllBytes($shortcutPath)
	$bytes[0x15] = $bytes[0x15] -bor 0x20 #set byte 21 (0x15) bit 6 (0x20) ON
	[System.IO.File]::WriteAllBytes($shortcutPath, $bytes)
}

function ensureAdmin {
	if (-NOT $(([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole] "Administrator"))) {
		Write-Output "Error: must be run as administrator."
		exit
	}
}

function installNode {
	# Install nodejs if its not already installed
	#Write-Output "Checking if there is a satisfactory nodejs installation... (this may produce a command not found error, you can ignore this!)"
	#$nodeVersion = node -v
	#if (!$nodeVersion) {
	#	Write-Output "Detected nodejs is not installed, downloading nodejs installer..."
	#	installNodeJs
	#}
}

#function installNodejs {
#	$arch = "x86";
#	if ($env:PROCESSOR_ARCHITECTURE -eq "AMD64") {
#		$arch = "x64"
#	}
#	$nodejsVersion = "v6.11.2"
#	$nodejsMsiUrl = "https://nodejs.org/dist/$nodejsVersion/node-$nodejsVersion-$arch.msi"
#	$nodejsMsiFilepath = FormatPath "$env:TEMP/nodejsv6_11_2.msi"
#	DownloadRemoteFile $nodejsMsiUrl $nodejsMsiFilepath
#
#	Write-Output "Installing nodejs..."
#	Start-Process msiexec.exe -Wait -ArgumentList "/I $nodejsMsiFilepath /quiet"
#	Write-Output "Finished installing nodejs..."
#	Remove-Item -Path $nodejsMsiFilepath -Force
#}

# Identify platform information and run any platform-specific logic
$platform = "win-ia32"
$arch = "ia32"
$nwjsVersion = "0.87.0"

if ($env:PROCESSOR_ARCHITECTURE -eq "AMD64") {
	$platform = "win-x64"
	$arch = "x64"
}

$nwjsFile = "nwjs-v$nwjsVersion-$platform"
$proprietaryDllFile = "$nwjsVersion-$platform.zip"
$nwjsFilepath = FormatPath "$rootPath/$nwjsFile.zip"
$proprietaryFilepath = FormatPath "$rootPath/$proprietaryDllFile"
#$nwjsUrl = "https://dl.nwjs.io/v$nwjsVersion/$nwjsFile.zip"
#$proprietaryUrl = "https://github.com/iteufel/nwjs-ffmpeg-prebuilt/releases/download/$nwjsVersion/$proprietaryDllFile"
# Download the correct versions of nwjs.zip
$nwjsUrl = "https://internal.keywesttechnology.com/breeze2/mirror/$nwjsFile.zip"
$proprietaryUrl = "https://internal.keywesttechnology.com/breeze2/mirror/$proprietaryDllFile"

function downloadNwjs {
	$isAdmin = ([Security.Principal.WindowsPrincipal] [Security.Principal.WindowsIdentity]::GetCurrent()).IsInRole([Security.Principal.WindowsBuiltInRole]::Administrator)
	$currentUser = [System.Security.Principal.WindowsIdentity]::GetCurrent().Name
	$logEntry = "Running as "
	if ($isAdmin) {
		$logEntry += "administrator"
	} else {
		$logEntry += "a non-administrator user"
	}
	$logEntry += " with username: $currentUser"
	Write-Output $logEntry

	Write-Output "Downloading nwjs"
	Write-Output "Platform: $platform, architecture: $arch, version: $nwjsVersion"

	Write-Output "Downloading nwjs $nwjsVersion for $platform..."

	DownloadRemoteFile $nwjsUrl $nwjsFilepath
	DownloadRemoteFile $proprietaryUrl $proprietaryFilepath
}

function installNwjs {
	Write-Output "Installing nwjs"
	
	# The nwjs zip file has a "nwjs-v$nwjsVersion-$platform" folder, so we unzip, move the contents, and delete the now empty folder
	ExtractZipFile $nwjsFilepath $rootPath
	Move-Item "$rootPath/$nwjsFile/*" $rootPath -Force
	ExtractZipFile $proprietaryFilepath $rootPath

	Remove-Item -Path $nwjsFilepath -Force
	Remove-Item -Path $proprietaryFilepath -Force
	Remove-Item -Path $rootPath/$nwjsFile -Force

	# Create chromium user data dir
	$userDataDir = FormatPath "$rootPath/chromium-config"
	New-Item -Force -ItemType directory -Path $userDataDir
}

# Compares two version strings against one another.
# Returns:
# 	-1 when versionA represents a lower version number than versionB
# 	 0 when versionA represents the same version number as versionB
# 	 1 when versionA represents a higher version number than versionB
function CompareVersions ([String]$versionA, [String]$versionB) {
	$tokensA = $versionA.Split(".")
	if ($tokensA.length -eq 3) {
		$tokensA += "0"
	}
	for ($itr = 0; $itr -lt $tokensA.length; ++$itr) {
		$tokensA[$itr] = [int]$tokensA[$itr];
	}
	$tokensB = $versionB.Split(".")
	if ($tokensB.length -eq 3) {
		$tokensB += "0"
	}
	for ($itr = 0; $itr -lt $tokensB.length; ++$itr) {
		$tokensB[$itr] = [int]$tokensB[$itr];
	}
	if ($tokensA[0] -gt $tokensB[0]) {
		return 1;
	} elseif ($tokensA[0] -eq $tokensB[0]) {
		if ($tokensA[1] -gt $tokensB[1]) {
			return 1;
		} elseif ($tokensA[1] -eq $tokensB[1]) {
			if ($tokensA[2] -gt $tokensB[2]) {
				return 1;
			} elseif ($tokensA[2] -eq $tokensB[2]) {
				if ($tokensA[3] -gt $tokensB[3]) {
					return 1;
				} elseif ($tokensA[3] -eq $tokensB[3]) {
					return 0;
				}
			}
		}
	}
	return -1;
}

# Required directories - do not modify these path values
$rootPath = FormatPath "$env:SystemDrive/breeze"

$watchdogPs1Path = FormatPath "$rootPath/watchdog.ps1"
$watchdogVbsPath = FormatPath "$rootPath/watchdog.vbs"
$startMonitorPath = FormatPath "$rootPath/start_monitor.ps1"
$stopMonitorPath = FormatPath "$rootPath/stop_monitor.ps1"
$startMonitorBatPath = FormatPath "$rootPath/start_monitor.bat"
$stopMonitorBatPath = FormatPath "$rootPath/stop_monitor.bat"
$explorerWatchdogPs1Path = FormatPath "$rootPath/watchdog_explorer.ps1"
$explorerWatchdogVbsPath = FormatPath "$rootPath/watchdog_explorer.vbs"

$desktopPath = FormatPath "$Home/Desktop"
$desktopShortcutPath = FormatPath "$desktopPath/BreezePlayer.lnk"
$stopBreezeShortcutPath = FormatPath "$desktopPath/StopBreeze.lnk"
$startBreezeShortcutPath = FormatPath "$desktopPath/StartBreeze.lnk"

$watchdogTaskName = "Breeze2WatchDog"
$rebootTaskName = "Breeze2AutoWatchDog"
$explorerWatchdogTaskName = "Breeze2ExplorerWatchDog"

$packagePath = FormatPath "$rootPath/package.json"

$configDirectory = FormatPath "$rootPath/config"
$updateLockPath = FormatPath "$configDirectory/update.lock"

$runlogPath = FormatPath "$configDirectory/run.log"
$errorlogPath = FormatPath "$configDirectory/err.log"
$updatelogPath = FormatPath "$configDirectory/update.log"