#!/bin/bash

# Assuming the user is breeze for now
user="breeze"
rootPath="/breeze"
configDirectory=$rootPath/config
updateLockPath=$configDirectory/update.lock

watchdogPath=$rootPath/watchdog.sh
runPath=$rootPath/run.sh

outlogPath=$configDirectory/out.log
errlogPath=$configDirectory/err.log
runlogPath=$configDirectory/run.log
updatelogPath=$configDirectory/update.log

function print_error {
	tput setab 0;
	tput setaf 1;
	echo "$@"
	tput sgr 0;
}

function print_info {
	tput setaf 2;
	echo "$@"
	tput sgr 0;
}

# Compares two version strings against one another.
# Returns:
# 	-1 when versionA represents a lower version number than versionB
# 	 0 when versionA represents the same version number as versionB
# 	 1 when versionA represents a higher version number than versionB
function CompareVersions {
	IFS='.' read -r -a tokensA <<< "$1"
	IFS='.' read -r -a tokensB <<< "$2"
	if [[ ${#tokensA[@]} -eq 3 ]] ; then
		tokensA[3]=0
	fi
	if [[ ${#tokensB[@]} -eq 3 ]] ; then
		tokensB[3]=0
	fi

	if [[ ${tokensA[0]} -gt ${tokensB[0]} ]] ; then 
		echo 1
		return 0
	elif [[ ${tokensA[0]} -eq ${tokensB[0]} ]] ; then
		if [[ ${tokensA[1]} -gt ${tokensB[1]} ]] ; then
			echo 1
			return 0
		elif [[ ${tokensA[1]} -eq ${tokensB[1]} ]] ; then
			if [[ ${tokensA[2]} -gt ${tokensB[2]} ]] ; then
				echo 1
				return 0
			elif [[ ${tokensA[2]} -eq ${tokensB[2]} ]] ; then
				if [[ ${tokensA[3]} -gt ${tokensB[3]} ]] ; then
					echo 1
					return 0
				elif [[ ${tokensA[3]} -eq ${tokensB[3]} ]] ; then
					echo 0
					return 0
				fi
			fi
		fi
	fi
	echo -1
}

function CreateLogs {
	cat > /etc/logrotate.d/breeze << EOL
$outlogPath {
	missingok
	notifempty
	compress
	size 10M
	rotate 12
}
$errlogPath {
	missingok
	notifempty
	compress
	size 10M
	rotate 12
}
$runlogPath {
	missingok
	notifempty
	compress
	size 10M
	rotate 12
}
$updatelogPath {
	missingok
	notifempty
	compress
	size 10M
	rotate 12
}
EOL
	chown $user:$user /etc/logrotate.d/breeze
}

function InstallNode {
	# lol, empty bash functions are not allowed. Insert a null statement:
	:;
	# Ensure nodejs is installed
	#if ! type npm >/dev/null 2>&1 ; then
	#	echo "Installing npm v0.33.2.."
	#	wget -qO- https://raw.githubusercontent.com/creationix/nvm/v0.33.2/install.sh | bash
	#	source ~/.bashrc
	#	echo "Installing nodejs v6.11.2.."
	#	nvm install 6.11.2
	#fi
}

function CopyFiles {
	print_info "Copying Breeze container files..."
	print_info "Delete all the files except the config folder"
	rm -r $rootPath/!(config)/
	print_info "Copying updated files"
	cp $updateDirectory/* $rootPath/
	chmod -R +x $rootPath/scripts
}

function InstallNwjs {
	platform="linux-ia32"
	arch="ia32"
	version="0.87.0"
	user="breeze"
	if [ -f /etc/os-release ]; then
		. /etc/os-release
	else
		print_error "Unable to detect file '/etc/os-release', assuming generic installation."
	fi
	if [ `getconf LONG_BIT` == "64" ]; then
		platform="linux-x64"
		arch="x64"
	fi
	# The latest nwjs build that is supported on 12.04, 12.10 and 13.10 is 22.3, which runs
	# Chromium v58.0.3029.110 and Node.js v7.10.0 (https://nwjs.io/blog/v0.22.3/)
	if [ $VERSION_ID == "12.04" ]; then
		version="0.22.3"
	fi
	if [ $VERSION_ID == "12.10" ]; then
		version="0.22.3"
	fi
	if [ $VERSION_ID == "13.10" ]; then
		version="0.22.3"
	fi

	# Download and install nwjs
	print_info "Downloading nwjs $version for $platform..."
	nwjsFile="nwjs-v$version-$platform"
	proprietaryDllFile="$version-$platform"
	cd $rootPath
	# Companies whitelist this domain and some firewalls will block other domains
	wget "https://internal.keywesttechnology.com/breeze2/mirror/$nwjsFile.tar.gz"
	wget "https://internal.keywesttechnology.com/breeze2/mirror/$proprietaryDllFile.zip"
	# wget "https://dl.nwjs.io/v$version/$nwjsFile.tar.gz"
	# wget "https://github.com/iteufel/nwjs-ffmpeg-prebuilt/releases/download/$version/$proprietaryDllFile.zip"
	tar -xzf $rootPath/$nwjsFile.tar.gz -C $rootPath
	unzip -o $rootPath/$proprietaryDllFile.zip -d $rootPath/lib
	rm -rf $rootPath/$nwjsFile.tar.gz
	rm -rf $rootPath/$proprietaryDllFile.zip

	# The CopyFiles deleted the old chromium cache directory, which is required for upgrading nwjs sometimes anyways, so recreate the folder
	userDataDir="$rootPath/chromium-config"
	mkdir -p $userDataDir
}

function CreateWatchdog {
	print_info "Creating watchdog..."
	cat > $watchdogPath << EOL
#!/bin/bash
[[ -e $updateLockPath ]] && exit;
cd $rootPath
if [ ! "\$(pidof nw)" ]; then
	export DISPLAY=$DISPLAY
	echo -e "\$(date)\tDetected container was not running... launching container..." >> $runlogPath
	sudo $rootPath/nw 1>>$outlogPath 2>>$errlogPath &
fi
EOL
	chmod +x $watchdogPath
	chown $user:$user $watchdogPath

	cat > $runPath << EOL
#!/bin/bash
[[ -e $updateLockPath ]] && exit;
cd $rootPath
if [ ! "\$(pidof nw)" ]; then
	export DISPLAY=$DISPLAY
	echo -e "\$(date)\tManually launching container from desktop icon..." >> $runlogPath
	sudo $rootPath/nw 1>>$outlogPath 2>>$errlogPath &
else
	echo -e "\$(date)\tAn attempt was made to manually launch the container from the desktop icon, but the container is already running!  Attempting to switch to Breeze Player..." >> $runlogPath
	xdotool search --name "Breeze Player" windowactivate
fi
EOL
	chmod +x $runPath
	chown $user:$user $runPath

	monitorCron="echo \"* * * * * $user $watchdogPath >> $runlogPath 2>&1\" > /etc/cron.d/breezecron"
	eval "$monitorCron"

	stopMonitorPath=$rootPath/stop_monitor.sh
	cat > $stopMonitorPath << EOL
#!/bin/bash
sudo rm /etc/cron.d/breezecron
EOL
	chmod +x $stopMonitorPath
	chown $user:$user $stopMonitorPath

	startMonitorPath=$rootPath/start_monitor.sh
	cat > $startMonitorPath << EOL
#!/bin/bash
nohup $monitorCron
nohup /bin/bash $watchdogPath
EOL
	chmod +x $startMonitorPath
	chown $user:$user $startMonitorPath
}

function CreateShortcuts {
	print_info "Creating desktop shortcuts..."
	desktopShortcut=/home/$user/Desktop/BreezePlayer.desktop
	# Remove old desktop icon, this was changed so there is now only a start and stop shortcut
	rm $desktopShortcut

	stopBreezeShortcut=/home/$user/Desktop/StopBreeze.desktop
	cat > $stopBreezeShortcut << EOL
[Desktop Entry]
Version=1.0
Name=StopBreeze
Comment=Prevents the Breeze 2.x Player from autolaunching.
Exec=$stopMonitorPath
Icon=$rootPath/icons/breeze_monitor_stop_icon.png
Terminal=false
Type=Application
Categories=Application;
EOL
	chmod +x $stopBreezeShortcut
	chown $user:$user $stopBreezeShortcut

	startBreezeShortcut=/home/$user/Desktop/StartBreeze.desktop
	cat > $startBreezeShortcut << EOL
[Desktop Entry]
Version=1.0
Name=StartBreeze
Comment=Allows the Breeze 2.x Player to autolaunch.
Exec=bash -c 'sudo $startMonitorPath'
Icon=$rootPath/icons/breeze_monitor_start_icon.png
Terminal=true
Type=Application
Categories=Application;
EOL
	chmod +x $startBreezeShortcut
	chown $user:$user $startBreezeShortcut
}