#!/bin/bash
echo "Multihead Configuration Script..."
echo "------------------------------------------"
echo "Common setups"
echo "[1920x1080 panels]"
echo "6x1 - 11520x1080"
echo "4x1 - 7680x1080"
echo "3x2 - 5760x2160"
echo "2x2 - 3840x2160"
echo "------------------------------------------"
echo "Enter width (e.g. 5760): "
read width
echo "Enter height (e.g. 2160): "
read height
echo "Enter columns (e.g. 3): "
read cols
echo "Enter rows (e.g. 2): "
read rows

xrandrArgs=""
configFile=/etc/X11/xorg.conf
rm $configFile
for ((y=0;y<rows;y++)); do
	for ((x=0;x<cols;x++)); do
		id=$(($x + $y * $cols))
		if [[ $x -eq 0 ]] && [[ $y -eq 0 ]]; then
			echo 'Section "Monitor"' >> $configFile
			echo '    Identifier "DisplayPort-0"' >> $configFile
			echo 'EndSection' >> $configFile
			xrandrArgs="--output DisplayPort-0 --auto"
		elif [[ $x -eq 0 ]]; then
			prevAnchorId=$((($y - 1) * $cols))
			echo 'Section "Monitor"' >> $configFile
			echo '    Identifier "DisplayPort-'"$id"'"' >> $configFile
			echo '    Option "Below" "DisplayPort-'"$prevAnchorId"'"' >> $configFile
			echo 'EndSection' >> $configFile
			xrandrArgs="$xrandrArgs --output DisplayPort-$id --auto --below DisplayPort-$prevAnchorId"
		else
			prevId=$(($id - 1))
			echo 'Section "Monitor"' >> $configFile
			echo '    Identifier "DisplayPort-'"$id"'"' >> $configFile
			echo '    Option "RightOf" "DisplayPort-'"$prevId"'"' >> $configFile
			echo 'EndSection' >> $configFile
			xrandrArgs="$xrandrArgs --output DisplayPort-$id --auto --right-of DisplayPort-$prevId"
		fi
	done
done

cat >>$configFile << EOL
Section "Screen"
	Identifier "Default-Screen"
	SubSection "Display"
		Virtual $width $height
		Depth 24
	EndSubSection
EndSection

Section "ServerLayout"
	Identifier "Default-ServerLayout"
	Screen "Default-Screen"
EndSection
EOL

rm /home/breeze/.config/monitors.xml
#dconf write /org/compiz/profiles/unity/plugins/core/detect-outputs true
#dconf update
dconf write /org/compiz/profiles/unity/plugins/core/detect-outputs false
gsettings set org.compiz.core:/org/compiz/profiles/unity/plugins/core/ outputs "['${width}x${height}+0+0']"

xrandr $xrandrArgs

echo "Set virtual resolution to: $widthx$height"
echo "Multi-Head setup finished.  Open the Display Settings panel and verify the outputs are correct, and then APPLY THE CONFIGURATION."
echo "If you do not open the Display Settings panel and apply the configuration, the monitors will NOT be configured correctly on reboot."
